#ifndef V1_H_
#define V1_H_


#include <eoOp.h>
/**
 * @class eoCPTEO_V1Mutation
 * @author Jacques Bibai
 * @brief definition d'un operateur de mutation : mutation permettant de modifier le second argument 
 * d'un fluent (arite du predicat = 2).  
 */

template<class GenotypeT> 
class eoCPTEO_V1Mutation: public eoMonOp<GenotypeT>
{
public:
  /**
 * Constructeur vide
 */
  eoCPTEO_V1Mutation()
  
  {
  
  }

 
  string className() const { return "eoCPTEO_V1Mutation"; }

   
 /**
   * @fn bool operator()(GenotypeT &)
   * @brief Operateur decrivant la maniere d'effectuer une mutation par modification du second argument
   * d'un fluent appartenant a une gare intermediaire. 
   * Ici on tire aleatoirement la gare et le fluent. Ensuite on tire aleatoirement dans la liste
   * des termes possible avec la contrainte : le fluent ainsi forme ne doit pas etre "mutex" avec les autres fluents
   * decrivant la gare   
   * @param individu instance de la classe GenotypeT
   * @return bool : vrai si l'operation a reussi et faux sinon
   */ 
  bool operator()(GenotypeT & _genotype) 
  {
    static bool isModified=false,testMut;
    //cout <<"******************************BREAKSTATION*******************************\n"<< _genotype.getBreakStation();
     for(size_t y=0; y<_genotype.getBreakStation();y++)
    {
    	if(y<_genotype.getGenome().size())
    	if(rng.flip(global.getpMeanAveragePerMutation()/_genotype.getGenome().size()))
    	{
		    unsigned idFluent,position1,position2, test=0;
		    //choix de la gare et du fluent
		  	position1=y;
		  	if(_genotype.get(position1).size()>0){
		  		position2=rng.random(_genotype.get(position1).size());
		  		idFluent=_genotype.get(position1).get(position2);
		 		// identification du predicat associe
				CPTPredicateArray preds = cptGetPredicates();
				unsigned idpred;
				for(size_t j=0;j<(unsigned)preds.nb;j++)
				{
					if(strcmp(cptGetPredicateName(preds.elems[j]),
					cptGetPredicateName(cptGetPredicate(cptGetFluentFromIndex(idFluent))))==0)
					{
						idpred=j;break;
					}
				}
				// choix du terme a modifier
				int modif=cptGetArity(preds.elems[idpred])-1;
				if(modif==-1)return false;
				//sauvegarde des autres arguments du fluent
				CPTTermArray terms = cptGetTerms(cptGetFluentFromIndex(idFluent));
				CPTTermArray val = cptGetPossibleTerms(preds.elems[idpred], modif);
				unsigned choix;
				testMut=false;
				do
				{
					//if(terms.nb<2)break;
					//choix aleatoire d'un terms
					choix=rng.random(val.nb);
					// v$-3õ½rification et construction du fluent corresspondant
					if(strcmp(cptGetTermName(val.elems[choix]),cptGetTermName(terms.elems[modif])))
					{
						int memeNom=1;
						for(size_t i=0;i<(unsigned)terms.nb;i++){
							if(i!=(unsigned)(modif))
							if(strcmp(cptGetTermName(val.elems[choix]),cptGetTermName(terms.elems[i])))
							{
								memeNom=0; break;
							}
						}
						if(memeNom){
							//construction du fluent correspondant
							std::string tmp("(");
							tmp+=cptGetPredicateName(preds.elems[idpred]);
							for(size_t i=0;i<(unsigned)terms.nb;i++)
							{
								tmp+=" ";
								if(i!=(unsigned)(modif))tmp+= cptGetTermName(terms.elems[i]);
								else tmp+=cptGetTermName(val.elems[choix]);
							}
							tmp+=")";
						
							CPTFluent fluent=cptGetFluentFromName((char *)tmp.c_str());
					 		if(fluent!=NULL){;
						 		// fin construction du fluent 
							 	idFluent=cptGetIndexFromFluent(fluent);
						 
						 		// test : mutuellement exclusifs
						 		if(_genotype.get(position1).mutex(idFluent,position2)==0)
						 		{
									//mise $-3õ½ jour du fluent 
						 			_genotype.get(position1).set(position2,idFluent);
						 			testMut=true;
						 		}
					 	
				 			}
						}
					}
				
					test++;
				
				}while (!testMut && !test==10);
		  	}
			if(!isModified && testMut)isModified=true;
    	}
    }
    if(!isModified)
    {
    		unsigned idFluent,position1,position2, test=0;
		    //choix de la gare et du fluent
		    if(_genotype.getBreakStation()<_genotype.getGenome().size())
		  	position1=rng.random(_genotype.getBreakStation());
		  	else position1=0;
		  	if(_genotype.get(position1).size()>0){
		  		position2=rng.random(_genotype.get(position1).size());
		  		idFluent=_genotype.get(position1).get(position2);
		  		// identification du predicat associe
				CPTPredicateArray preds = cptGetPredicates();
				unsigned idpred;
				//cout << _genotype; 
				for(size_t j=0;j<(unsigned)preds.nb;j++)
				{
					if(strcmp(cptGetPredicateName(preds.elems[j]),
					cptGetPredicateName(cptGetPredicate(cptGetFluentFromIndex(idFluent))))==0)
					{
						idpred=j;break;
					}
				}
				// choix du terme a modifier
				int modif=(int)cptGetArity(preds.elems[idpred])-1;
				CPTTermArray terms = cptGetTerms(cptGetFluentFromIndex(idFluent));
				CPTTermArray val = cptGetPossibleTerms(preds.elems[idpred], modif);
				unsigned choix;
				do
				{
				
					//if(terms.nb<2)break;
					//choix aleatoire d'un terms
					choix=rng.random(val.nb);
					// v$-3õ½rification et construction du fluent corresspondant
					if(strcmp(cptGetTermName(val.elems[choix]),cptGetTermName(terms.elems[modif])))
					{
						int memeNom=1;
						for(size_t i=0;i<(unsigned)terms.nb;i++){
							if(i!=(unsigned)(modif))
							if(strcmp(cptGetTermName(val.elems[choix]),cptGetTermName(terms.elems[i])))
							{
								memeNom=0; break;
							}
						}
						if(memeNom){
						//construction du fluent correspondant
							std::string tmp("(");
							tmp+=cptGetPredicateName(preds.elems[idpred]);
							for(static size_t i=0;i<(unsigned)terms.nb;i++)
							{
								tmp+=" ";
								if(i!=(unsigned)(modif))tmp+= cptGetTermName(terms.elems[i]);
								else tmp+=cptGetTermName(val.elems[choix]);
							}
							tmp+=")";
							
							CPTFluent fluent=cptGetFluentFromName((char *)tmp.c_str());
						 	if(fluent!=NULL){
							 	// fin construction du fluent 
								 idFluent=cptGetIndexFromFluent(fluent);
							 
							 	// test : mutuellement exclusifs
							 	if(_genotype.get(position1).mutex(idFluent,position2)==0)
							 	{
									//mise $-3õ½ jour du fluent 
							 		_genotype.get(position1).set(position2,idFluent);
							 		isModified=true;
							 	}
						 	}
						}
					}
				
					test++;
				
				}while (!isModified && !test==10);
    		}
		}
	//construction et modification du fluent
	
    return isModified;
   
  }

private:

};

#endif /*V1_H_*/
